"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.log = exports.isEntryFunc = exports.isRegExp = exports.isEntryObject = exports.isEntryDescription = exports.isEntryItem = exports.isEntry = exports.isPlainObject = exports.isString = exports.isType = exports.hasOwn = void 0;
const chalk = require("chalk");
const hasOwn = (val, key) => Object.prototype.hasOwnProperty.call(val, key);
exports.hasOwn = hasOwn;
const isType = (type) => (val) => {
    return Object.prototype.toString.call(val) === `[object ${type}]`;
};
exports.isType = isType;
const isString = (val) => {
    return typeof val === 'string';
};
exports.isString = isString;
const isPlainObject = (val) => {
    return val !== null && typeof val === 'object';
};
exports.isPlainObject = isPlainObject;
// only for webpack@4.x
const isEntry = (val) => {
    if ((0, exports.isPlainObject)(val)) {
        return Object.values(val).every((value) => {
            return ((0, exports.isString)(value) ||
                (Array.isArray(value) && value.every((val) => (0, exports.isString)(val))));
        });
    }
    return false;
};
exports.isEntry = isEntry;
// for webpack 4
const isEntryItem = (val) => {
    return (0, exports.isString)(val) || (Array.isArray(val) && val.every((e) => (0, exports.isString)(e)));
};
exports.isEntryItem = isEntryItem;
// work in webpack 5
const isEntryDescription = (val) => {
    return (0, exports.isPlainObject)(val) && (0, exports.hasOwn)(val, 'import');
};
exports.isEntryDescription = isEntryDescription;
const isEntryObject = (val) => {
    if ((0, exports.isPlainObject)(val)) {
        return Object.values(val).every((innerVal) => {
            return (0, exports.isEntryItem)(innerVal) || (0, exports.isEntryDescription)(innerVal);
        });
    }
    return false;
};
exports.isEntryObject = isEntryObject;
const isRegExp = (val) => {
    return (0, exports.isPlainObject)(val) && (0, exports.isType)('RegExp')(val);
};
exports.isRegExp = isRegExp;
const isEntryFunc = (val) => {
    return typeof val === 'function';
};
exports.isEntryFunc = isEntryFunc;
exports.log = {
    warn: (msg) => {
        console.warn(chalk.red(msg));
    },
};
